
--[[ Libraries ]]
require('util')

--[[ Classes ]]
CSV = require('csv')()

--[[ Initialization ]]
local function UpdateGlobals()
end

-- First run
script.on_init(UpdateGlobals)

-- Every run
script.on_load(function()

end)

-- Configs changed
script.on_configuration_changed(UpdateGlobals)

--[[ Utilities ]]
function wtf(...)
    local args = {...}
    for k,v in pairs(args) do
        if type(v) ~= 'string' or v == nil then
            args[k] = stringify(v)
        end
    end
    for k,v in pairs(game.players) do
        v.print(table.concat(args, ', '))
    end
end

--[[ Events ]]

-- Settings
local tickrate = 600 -- once every ten seconds
local updatesPerStep = 2
local options = {
    {
        power = 1000000, -- 1MW (1MW less than generated)
        description = "Not using accumulators",
    },
    {
        power = 3000000, -- 3MW (1MW more than generated)
        description = "Using accumulators",
    }
}

-- State
local running = false
local wasRunning = false
local option = 0
local updates = 0
local powerExpected = nil

-- Game updates
script.on_event(defines.events.on_tick, function(event)

    -- Only at [tickrate] ticks
    if running and (event.tick % tickrate == 0) then

        -- Write result if we were already running
        if powerExpected ~= nil then
            CSV:Value('received', global.consumer.energy)
            CSV:Value('expected', powerExpected)
            CSV:Value('receive%', global.consumer.energy / powerExpected * 100)
            CSV:Value('tickAfter', game.tick)
            CSV:Next()
        end

        -- Toggle state
        option = option + 1
        if option > #options then
            option = 1
        end

        -- Load next option and calculate the expected energy
        local opt = options[option]
        powerExpected = opt.power / 60 * tickrate

        -- Notify player
        wtf(opt.description)

        -- Reset consumer, using current option
        global.consumer.energy = 0
        global.consumer.electric_buffer_size = powerExpected * 10 -- double buffer size to see the overflow
        global.consumer.electric_input_flow_limit = powerExpected / tickrate

        -- Write CSV values
        CSV:Value('tickrate', tickrate)
        CSV:Value('tickBefore', game.tick)
        CSV:Value('buffer', global.consumer.electric_buffer_size)
        CSV:Value('flow', global.consumer.electric_input_flow_limit)
    end

end)

script.on_event(defines.events.on_player_joined_game, function(event)

    local ply = game.players[event.player_index]

    -- Power consumer
    global.consumer = game.surfaces['nauvis'].create_entity({
        name = "test-input",
        force = "player",
        position = {x=0, y=-1},
    })
    --global.consumer.electric_buffer_size = 10000000 -- 10MW    

    -- Power provider
    global.provider = game.surfaces['nauvis'].create_entity({
        name = "test-output",
        force = "player",
        position = {x=0, y=1},
    })

    -- Accumulator
    global.accumulator = game.surfaces['nauvis'].create_entity({
        name = "test-accumulator",
        force = "player",
        position = {x=1, y=1},
    })
    global.accumulator.electric_buffer_size = 100000000 -- 100MW
    global.accumulator.energy = global.accumulator.electric_buffer_size
    global.accumulator.electric_input_flow_limit = 10000000 -- 10MW
    global.accumulator.electric_output_flow_limit = 10000000 -- 10MW

    -- Pole to connect them
    game.surfaces['nauvis'].create_entity({
        name = "medium-electric-pole",
        force = "player",
        position = {x=0, y=0},
    })

    ply.cheat_mode = true
    ply.force.research_all_technologies()

    running = true
end)

-- Chat for debugging
script.on_event(defines.events.on_console_chat, function(event)
    CSV:Write('test') -- save to script_output/csv/test.csv
    -- only works if file is writable! e.g. not open in Excel
end)